@@ -1,21 +1,21 @@
 /*
-    Copyright 2005-2015 Intel Corporation.  All Rights Reserved.
+    Copyright (c) 2005-2016 Intel Corporation
+
+    Licensed under the Apache License, Version 2.0 (the "License");
+    you may not use this file except in compliance with the License.
+    You may obtain a copy of the License at
+
+        http://www.apache.org/licenses/LICENSE-2.0
+
+    Unless required by applicable law or agreed to in writing, software
+    distributed under the License is distributed on an "AS IS" BASIS,
+    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+    See the License for the specific language governing permissions and
+    limitations under the License.
+
+
 
-    This file is part of Threading Building Blocks. Threading Building Blocks is free software;
-    you can redistribute it and/or modify it under the terms of the GNU General Public License
-    version 2  as  published  by  the  Free Software Foundation.  Threading Building Blocks is
-    distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the
-    implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
-    See  the GNU General Public License for more details.   You should have received a copy of
-    the  GNU General Public License along with Threading Building Blocks; if not, write to the
-    Free Software Foundation, Inc.,  51 Franklin St,  Fifth Floor,  Boston,  MA 02110-1301 USA
 
-    As a special exception,  you may use this file  as part of a free software library without
-    restriction.  Specifically,  if other files instantiate templates  or use macros or inline
-    functions from this file, or you compile this file and link it with other files to produce
-    an executable,  this file does not by itself cause the resulting executable to be covered
-    by the GNU General Public License. This exception does not however invalidate any other
-    reasons why the executable file might be covered by the GNU General Public License.
 */
 
 #ifndef __TBB__flow_graph_types_impl_H
@@ -361,7 +361,7 @@ namespace internal {
 #endif /* __TBB_CPP11_RVALUE_REF_PRESENT */
 
         ~initializer_list_wrapper() {
-            if ( my_begin ) 
+            if ( my_begin )
                 free( my_begin );
         }
 
@@ -451,12 +451,12 @@ public:
     explicit Wrapper( const T& other ) : value_space(other) { }
     explicit Wrapper(const Wrapper& other) : value_space(other.value_space) { }
 
-    /*override*/void CopyTo(void* newSpace) const {
+    void CopyTo(void* newSpace) const __TBB_override {
         _unwind_space guard((pointer_type)newSpace);
         (void) new(newSpace) Wrapper(value_space);
         guard.space = NULL;
     }
-    /*override*/~Wrapper() { }
+    ~Wrapper() { }
 };
 
 // specialization for array objects
@@ -519,11 +519,11 @@ public:
         guard.space = NULL;
     }
 
-    /*override*/void CopyTo(void* newSpace) const {
+    void CopyTo(void* newSpace) const __TBB_override {
         (void) new(newSpace) Wrapper(*this);  // exceptions handled in copy constructor
     }
 
-    /*override*/~Wrapper() {
+    ~Wrapper() {
         // have to destroy explicitly in reverse order
         pointer_type vp = reinterpret_cast<pointer_type>(&value_space);
         for(size_t i = N; i > 0 ; --i ) vp[i-1].~value_type();
@@ -706,6 +706,8 @@ const V& cast_to(T const &t) { return t.template cast_to<V>(); }
 template<typename V, typename T>
 bool is_a(T const &t) { return t.template is_a<V>(); }
 
+enum op_stat { WAIT = 0, SUCCEEDED, FAILED };
+
 }  // namespace internal
 
 #endif  /* __TBB__flow_graph_types_impl_H */
